/*
	ProWesS, PROGS Windowing System

	PWChange (and PWDoChange)

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 14 January 1995
	06.01.1996 ActionInit support added
	16.04.1997 PW_OBJECT_POINTER_START added
*/

#include <sms.h>
#include <ctype.h>

#include "private.h"
#include "PWHandler.h"

Error PWDoChange(PWObject object, int *taglist)
{
	int *orgtaglist=taglist;
	pt xfac, yfac;
	PWSystem *system;

	if (object==NULL) return ERR_IPAR;
	system=object->event.region->system;

	/* process argument list */
	while (*taglist)
	{
		char tag_recognized=TRUE;

		switch (*taglist)
		{
		case PW_OBJECT_AUXILIARY:
			object->auxiliary=(void *)(*(taglist+1));
			break;
		case PW_OBJECT_POINTER_START:
			object->status|=OSTATUS_POINTER;
			break;
		case PW_OBJECT_AUTOSIZE:
			object->status|=OSTATUS_AUTOSIZE;
			break;

		case PW_GLOBAL_AUXILIARY:
			system->global=(void *)(*(taglist+1));
			break;
		case PW_SYSTEM_BREAKDOWN:
			system->status|=SSTATUS_BREAKDOWN;
			break;
		case PW_SYSTEM_SLEEP:
			system->status|=SSTATUS_BREAKDOWN|SSTATUS_SLEEP;
			break;
		case PW_POINTER:
			system->subwindows.main.sprite=(Sprite *)(*(taglist+1));
			break;
		case PW_WINDOWFIT:
			if (object->type->event_type!=PW_TYPE_EVENT_REGION)
				return ERR_IPAR;
			object->event.region->windowfit=(char)(*(taglist+1));
			break;
		case PW_SCALE_FACTOR:
			if (object->type->event_type!=PW_TYPE_EVENT_REGION)
				return ERR_IPAR;
			object->event.region->scale=(Scale)(*(taglist+1));
			break;
		case PW_WINDOW_BUTTON:
			system->status|=SSTATUS_BUTFRAME|SSTATUS_WINCHANGE;
			break;
		case PW_WINDOW_NOSCALE:
			system->status|=SSTATUS_NOXSCALE|SSTATUS_NOYSCALE;
			break;
		case PW_WINDOW_NOXSCALE:
			system->status|=SSTATUS_NOXSCALE;
			break;
		case PW_WINDOW_NOYSCALE:
			system->status|=SSTATUS_NOYSCALE;
			break;
		case PW_CURSOR_SEPARATE:
			system->subwindows.main.cursor=(*(taglist+1));
			break;

		case PW_WINDOW_SIZE_PIX:
			system->status|=SSTATUS_WINCHANGE;
			system->curr_xsiz=(*(taglist+1));
			system->curr_ysiz=(*(taglist+2));
			break;
		case PW_WINDOW_XSIZE_PIX:
			system->status|=SSTATUS_WINCHANGE;
			system->curr_xsiz=(*(taglist+1));
			break;
		case PW_WINDOW_YSIZE_PIX:
			system->status|=SSTATUS_WINCHANGE;
			system->curr_ysiz=(*(taglist+1));
			break;
		case PW_WINDOW_SIZE:
			system->status|=SSTATUS_WINCHANGE;
			system->curr_xsiz=PF2xpix(*(taglist+1));
			system->curr_ysiz=PF2ypix(*(taglist+2));
			break;
		case PW_WINDOW_XSIZE:
			system->status|=SSTATUS_WINCHANGE;
			system->curr_xsiz=PF2xpix(*(taglist+1));
			break;
		case PW_WINDOW_YSIZE:
			system->status|=SSTATUS_WINCHANGE;
			system->curr_ysiz=PF2ypix(*(taglist+1));
			break;

		case PW_WINDOW_ORIGIN_RESET:
			system->status|=SSTATUS_WINCHANGE;
			system->win_xpos=system->win_ypos=-1;
			break;
		case PW_WINDOW_ORIGIN_PIX:
			system->status|=SSTATUS_WINCHANGE;
			system->win_xpos=(*(taglist+1));
			system->win_ypos=(*(taglist+2));
			break;
		case PW_WINDOW_XORIGIN_PIX:
			system->status|=SSTATUS_WINCHANGE;
			system->win_xpos=(*(taglist+1));
			break;
		case PW_WINDOW_YORIGIN_PIX:
			system->status|=SSTATUS_WINCHANGE;
			system->win_ypos=(*(taglist+1));
			break;
		case PW_WINDOW_ORIGIN:
			system->status|=SSTATUS_WINCHANGE;
			system->win_xpos=PF2xpix((*(taglist+1)));
			system->win_ypos=PF2ypix((*(taglist+2)));
			break;
		case PW_WINDOW_XORIGIN:
			system->status|=SSTATUS_WINCHANGE;
			system->win_xpos=PF2xpix((*(taglist+1)));
			break;
		case PW_WINDOW_YORIGIN:
			system->status|=SSTATUS_WINCHANGE;
			system->win_ypos=PF2ypix((*(taglist+1)));
			break;

		case PW_KEYPRESS:
			{
				char key=(char)(*(taglist+1));

				if (object->type->event_type!=PW_TYPE_EVENT_KEYPRESS)
					return ERR_IPAR;
				object->event.keypress->key=key;
				object->event.keypress->alternative=islower(key)?toupper(key):tolower(key);
			}
			break;

		case PW_CATCH_OBJECT:
			if (system->activated && system->catchkeys &&
				system->catchkeys->type->PWHandle)
				system->catchkeys->type->PWHandle(system->catchkeys,PW_EVENT_CATCHEND);
			system->catchkeys=(PWObject)(*(taglist+1));
			if (system->activated && system->catchkeys &&
				system->catchkeys->type->PWHandle)
				system->catchkeys->type->PWHandle(system->catchkeys,PW_EVENT_CATCHSTART);
			break;

		case PW_SYSTEM_ACTION_INIT:
			system->ActionInit=(Error (*)(PWObject))(*(taglist+1));
			break;
		case PW_SYSTEM_ACTION_EVENT1:
			system->EventLevel1=(Error (*)(PWObject))(*(taglist+1));
			break;
		case PW_SYSTEM_ACTION_EVENT2:
			system->EventLevel2=(Error (*)(PWObject))(*(taglist+1));
			break;
		case PW_SYSTEM_ACTION_EVENT3:
			system->EventLevel3=(Error (*)(PWObject))(*(taglist+1));
			break;
		case PW_SYSTEM_ACTION_EVENT4:
			system->EventLevel4=(Error (*)(PWObject))(*(taglist+1));
			break;
		case PW_SYSTEM_ACTION_EVENT5:
			system->EventLevel5=(Error (*)(PWObject))(*(taglist+1));
			break;
		case PW_SYSTEM_ACTION_EVENT6:
			system->EventLevel6=(Error (*)(PWObject))(*(taglist+1));
			break;
		case PW_SYSTEM_ACTION_EVENT7:
			system->EventLevel7=(Error (*)(PWObject))(*(taglist+1));
			break;
		case PW_SYSTEM_ACTION_EVENT8:
			system->EventLevel8=(Error (*)(PWObject))(*(taglist+1));
			break;

		default:
			tag_recognized=FALSE;
			break;
		}

		if (tag_recognized)
		{
			*taglist&=PW_TAGMASK_SKIP;
		}

		
		taglist=PWSkipTag(taglist);
	}

	/* let the object parse the remaining tags */
	if (object->type->PWChange)
		return object->type->PWChange(object, orgtaglist);
	else
		return ERR_OK;
}

Error PWChange(PWObject object, ...)
{
	return PWDoChange(object,(int *)((&object)+1));
}
