/*
	ProWesS, PROGS Windowing System

	PWScrollScreen

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 10 July 1995
	20.10.1995 window origin now correct (as from PageOrigin in PROforma)
	04.02.1996 support for adjusting pointer position while scrolling
			   made sure scrolling can't crash if current region has no owner
	08.04.1996 minimum scrolling distance could be larger than screen, fixed
	21.04.1997 fixed a problem which was introduced because PFPageShow only
			   displays the bit inside the PageBbox
*/

#include "private.h"
#include "PWHandler.h"

extern short system_borderwidth;
extern short system_bordercolour;
extern short system_scaleborderwidth;
extern short system_scalebordercolour;
extern pt	 system_scroll_distance;
extern pt	 Xpixel2point, Ypixel2point;
extern short XWindowSize, YWindowSize;

Error PWScrollTouch(PWSystem *system)
{
	pix xloff=system->ptr_xpos;
	pix xroff=XWindowSize-system->ptr_xpos;
	pix ytoff=system->ptr_ypos;
	pix yboff=YWindowSize-system->ptr_ypos;

	if (min(xloff,xroff)<min(ytoff,yboff))
	{
		if (system->status&SSTATUS_XSCROLLABLE)
			PWScrollScreen(system,(xloff<xroff)?-1:1,0,TRUE);
	}
	else
	{
		if (system->status&SSTATUS_YSCROLLABLE)
			PWScrollScreen(system,0,(ytoff<yboff)?-1:1,TRUE);
	}
}

Error PWScrollScreen(PWSystem *system, int x, int y, char move)
{
	pix xorg, yorg, dist;
	PWRegion *active=system->active;
	Error (*ActivateEntry)(PWObject);
	short xpos, ypos;

	if (x)
	{
		/* see if scrolling further is possible */
		if (x>0 && system->xorg>system->min_xsiz-system->curr_xsiz)
			return ERR_OK;

		/* determine maximum scrolling distance */
		dist=min(PF2xpix(system_scroll_distance), system->curr_xsiz*3/4);

		xorg=system->xorg+( x>0 ? dist : -dist);
		if (xorg>system->min_xsiz-system->curr_xsiz)
			xorg=system->min_xsiz-system->curr_xsiz;
		if (xorg<0) xorg=0;

		xorg=system->xorg-xorg;
		yorg=0;
	}
	if (y)
	{
		/* see if scrolling further is possible */
		if (y>0 && system->yorg>system->min_ysiz-system->curr_ysiz)
			return ERR_OK;

		/* determine maximum scrolling distance */
		dist=min(PF2ypix(system_scroll_distance), system->curr_ysiz*3/4);

		yorg=system->yorg+( y>0 ? dist : -dist);
		if (yorg>system->min_ysiz-system->curr_ysiz)
			yorg=system->min_ysiz-system->curr_ysiz;
		if (yorg<0) yorg=0;

		yorg=system->yorg-yorg;
		xorg=0;
	}

	if (xorg || yorg)
	{
		int pxsiz, pysiz, i;
		Window win=system->window;
		SubWindow *sw=system->subwin;

		/* initialise the variables */
		pxsiz=system->curr_xsiz;
		pysiz=system->curr_ysiz;

		PFPPageScroll(system->gstate,xorg,yorg);
		/* adjust pageorigin for accurate pointer position */
		system->xorg-=xorg;
		system->yorg-=yorg;

		/* adjust pointer position */
		if (move)
		{
			WINPointerPositionGet(system->window,&xpos,&ypos);
			WINPointerPositionSet(system->window,xpos+xorg,ypos+yorg);
		}
		else
			/* force move pointer event */
			system->ptr_xpos=-1;

		/* need to redraw the borders */
		PWDrawBorders(system);

		/* and the contents of the screen */
		system->status|=SSTATUS_ALLDRAW;
		PWRedraw(system);
		if (active && active->owner &&
			(ActivateEntry=active->owner->type->PWRegionEntry))
			ActivateEntry(active->owner);
		PFPageBboxRestore(system->gstate);
		PFPageShow(system->gstate);
	}

	return ERR_OK;
}
